@@ -64,34 +64,146 @@ sub alien {
         my $pre   = Path::Tiny->cwd->child( qw[blib arch auto], $opt{meta}->name )->absolute;
         chdir $kid->absolute->stringify;
         warn Path::Tiny->cwd->absolute;
-        my $make = $opt{config}->get('make');
-        my $configure
-            = './configure --prefix=' . $pre->absolute . ' CFLAGS="-fPIC ' .
-            ( $opt{config}->get('osname') =~ /bsd/ ? '' : $CFLAGS ) . '" LDFLAGS="' .
-            ( $opt{config}->get('osname') =~ /bsd/ ? '' : $LDFLAGS ) . '"';
-        if ( $opt{config}->get('osname') eq 'MSWin32' ) {
-            require Devel::CheckBin;
-            for my $exe ( $make, qw[make nmake mingw32-make] ) {
-                next unless Devel::CheckBin::check_bin($exe);
-                $configure
-                    = '.\configure.bat /tool-gcc /make-' . ( $exe eq 'nmake' ? 'nmake' : 'make' );
-                $make = $exe;
-                last;
+        if (1) {
+            my $make = $opt{config}->get('make');
+            my $configure
+                = './configure --prefix=' . $pre->absolute . ' CFLAGS="-fPIC ' .
+                ( $opt{config}->get('osname') =~ /bsd/ ? '' : $CFLAGS ) . '" LDFLAGS="' .
+                ( $opt{config}->get('osname') =~ /bsd/ ? '' : $LDFLAGS ) . '"';
+            if ( $opt{config}->get('osname') eq 'MSWin32' ) {
+                require Devel::CheckBin;
+                for my $exe ( $make, qw[make nmake mingw32-make] ) {
+                    next unless Devel::CheckBin::check_bin($exe);
+                    $make      = $exe;
+                    $configure = '.\configure.bat /tool-' . $opt{config}->get('cc') . ' /make-';
+                    if ( $exe eq 'nmake' ) {
+                        $configure .= 'nmake';
+                        $make      .= ' -f Nmakefile';
+                    }
+                    else {
+                        $configure .= 'make';
+                        $make      .= ' CC=gcc VPATH=. PREFIX="' . $pre->absolute . '"';
+                    }
+                    last;
+                }
+                warn($_) && system($_ ) for $configure, $make;
+                my %libs = (
+                    dyncall => [
+                        qw[dyncall_version.h dyncall_macros.h dyncall_config.h
+                            dyncall_types.h dyncall.h dyncall_signature.h
+                            dyncall_value.h dyncall_callf.h dyncall_alloc.h
+                        ]
+                    ],
+                    dyncallback => [
+                        qw[dyncall_thunk.h dyncall_thunk_x86.h
+                            dyncall_thunk_ppc32.h dyncall_thunk_x64.h
+                            dyncall_thunk_arm32.h dyncall_thunk_arm64.h
+                            dyncall_thunk_mips.h dyncall_thunk_mips64.h
+                            dyncall_thunk_ppc64.h dyncall_thunk_sparc32.h
+                            dyncall_thunk_sparc64.h dyncall_args.h
+                            dyncall_callback.h
+                        ]
+                    ],
+                    dynload => [qw[dynload.h]],
+                );
+                $pre->child('include')->mkdir;
+                $pre->child('lib')->mkdir;
+                for my $lib ( keys %libs ) {
+
+                    #chdir $kid->child($lib)->absolute;
+                    warn $kid->child( $lib, 'lib' . $lib . '_s' . $opt{config}->get('_a') );
+                    $kid->child( $lib, 'lib' . $lib . '_s' . $opt{config}->get('_a') )
+                        ->copy( $pre->child('lib')->absolute );
+                    for ( @{ $libs{$lib} } ) {
+                        warn sprintf '%s => %s', $kid->child( $lib, $_ ),
+                            $pre->child( 'include', $_ )->absolute;
+                        warn $kid->child( $lib, $_ )
+                            ->copy( $pre->child( 'include', $_ )->absolute );
+                    }
+                }
+            }
+            else {
+                $make = $opt{config}->get('make');
+                warn($_) && system($_ ) for $configure, $make, $make . ' install';
             }
-            rename 'Makefile.generic',             'Makefile';
-            rename 'dyncall/Makefile.generic',     'dyncall/Makefile';
-            rename 'dynload/Makefile.generic',     'dynload/Makefile';
-            rename 'dyncallback/Makefile.generic', 'dyncallback/Makefile';
         }
-        else { $make = $opt{config}->get('make') }
-        warn($_) && system($_ )
-            for grep {defined} $configure,
-            $make . ' V=1' . ( $opt{config}->get('osname') eq 'MSWin32' ?
-                ' CC=gcc VPATH=. PREFIX="' . $pre->absolute . '"' : '' ),
-            $make . ' V=1' . ( $opt{config}->get('osname') eq 'MSWin32' ?
-                ' CC=gcc VPATH=. PREFIX="' . $pre->absolute . '"' : '' ) .
-            ' install';
-        warn Path::Tiny->cwd->absolute;
+        else {    # Future, maybe...
+            require ExtUtils::CBuilder;
+            my $builder = ExtUtils::CBuilder->new( config => ( $opt{config}->values_set ) );
+            $pre->child('lib')->mkdir;
+            $pre->child('include')->mkdir;
+            my %libs = (
+                dyncall => {
+                    c => [
+                        qw[dyncall_vector.c dyncall_api.c dyncall_callvm.c
+                            dyncall_callvm_base.c dyncall_call.S dyncall_callf.c
+                            dyncall_aggregate.c]
+                    ],
+                    h => [
+                        qw[dyncall_version.h dyncall_macros.h dyncall_config.h
+                            dyncall_types.h dyncall.h dyncall_signature.h
+                            dyncall_value.h dyncall_callf.h dyncall_alloc.h
+                        ]
+                    ]
+                },
+                dyncallback => {
+                    c => [
+                        qw[dyncall_alloc_wx.c dyncall_args.c dyncall_callback.c
+                            dyncall_callback_arch.S dyncall_thunk.c]
+                    ],
+                    h => [
+                        qw[dyncall_thunk.h dyncall_thunk_x86.h
+                            dyncall_thunk_ppc32.h dyncall_thunk_x64.h
+                            dyncall_thunk_arm32.h dyncall_thunk_arm64.h
+                            dyncall_thunk_mips.h dyncall_thunk_mips64.h
+                            dyncall_thunk_ppc64.h dyncall_thunk_sparc32.h
+                            dyncall_thunk_sparc64.h dyncall_args.h
+                            dyncall_callback.h
+                        ]
+                    ]
+                },
+                dynload => { c => [qw[dynload.c dynload_syms.c]], h => [qw[dynload.h]] },
+            );
+            #
+            for my $lib (qw[dyncall dyncallback dynload]) {
+                my @objs;
+                chdir $kid->child($lib)->absolute;
+                for my $c ( @{ $libs{$lib}{c} } ) {
+                    my $ob_file = $builder->compile(
+                        source => $c,
+
+                       #defines      => { VERSION => qq/"$version"/, XS_VERSION => qq/"$version"/ },
+                        include_dirs => [
+                            curdir, $kid->child('dyncall')->stringify,
+                            $pre->child('include')->stringify,
+                        ],
+
+                        #extra_compiler_flags => (
+                        #    '-fPIC ' . ( $opt{config}->get('osname') =~ /bsd/ ? '' : $CFLAGS ) .
+                        #        ( $DEBUG ? ' -ggdb3 ' : '' )
+                        #)
+                    );
+                    push @objs, $ob_file;
+                }
+                $builder->link(
+
+             #extra_linker_flags => (
+             #    ( $opt{config}->get('osname') =~ /bsd/ ? '' : $LDFLAGS ) . ' -L' .
+             #        dirname($source) . ' -L' . $pre->child( $opt{meta}->name, 'lib' )->stringify .
+             #        ' -ldyncall_s -ldyncallback_s -ldynload_s'
+             #),
+                    objects  => [@objs],
+                    lib_file => $pre->child( 'lib', 'lib' . $lib . '_s' . $opt{config}->get('_a') )
+                        ->stringify
+                );
+                for ( @{ $libs{$lib}{h} } ) {
+                    warn sprintf '%s => %s', $kid->child( $lib, $_ ),
+                        $pre->child( 'include', $_ )->absolute;
+                    warn $kid->child( $lib, $_ )->copy( $pre->child( 'include', $_ )->absolute );
+                }
+            }
+            #
+        }
         chdir $cwd->stringify;
     }
     else {
@@ -229,12 +341,16 @@ sub process_xs {
     my $builder = ExtUtils::CBuilder->new( config => ( $opt{config}->values_set ) );
     my $pre     = Path::Tiny->cwd->child(qw[blib arch auto])->absolute;
     my $obj     = $builder->object_file($c_file);
+    warn $pre->child( $opt{meta}->name, 'include' )->stringify;
     my $ob_file = $builder->compile(
         'C++'        => 1,
         source       => $c_file,
         defines      => { VERSION => qq/"$version"/, XS_VERSION => qq/"$version"/ },
-        include_dirs =>
-            [ curdir, dirname($source), $pre->child( $opt{meta}->name, 'include' )->stringify ],
+        include_dirs => [
+            curdir,                                                dirname($source),
+            $pre->child( $opt{meta}->name, 'include' )->stringify, dirname('dyncall/dyncall'),
+            dirname('dyncall/dynload'),                            dirname('dyncall/dyncallback')
+        ],
         extra_compiler_flags => (
             '-fPIC ' . ( $opt{config}->get('osname') =~ /bsd/ ? '' : $CFLAGS ) .
                 ( $DEBUG ? ' -ggdb3 ' : '' )
